//+------------------------------------------------------------------+
//|                                                      CCI Histo.mq4 |
//+------------------------------------------------------------------+
#property copyright "www.forex-station.com"
#property link      "www.forex-station.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  clrRoyalBlue
#property indicator_color2  clrRed
#property indicator_color3  clrMaroon
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property strict

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_tbiased2,   // Trend biased (extreme) price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased,  // Heiken ashi trend biased price
   pr_hatbiased2  // Heiken ashi trend biased (extreme) price
};

extern ENUM_TIMEFRAMES    TimeFrame         = PERIOD_CURRENT;    // Time frame to use
extern int                CCIPeriod         = 34;                // Cci period
extern enPrices           CCIPrice          = pr_typical;        // Cci price
extern double             OverSold          = -200;              // Oversold level
extern double             OverBought        = 200;               // Overbought level
extern bool               alertsOn          = true;              // Turn alerts on?
extern bool               alertsOnCurrent   = false;             // Alerts on current (still opened) bar?
extern bool               alertsMessage     = true;              // Alerts should display a message?
extern bool               alertsSound       = false;             // Alerts should play a sound?
extern bool               alertsNotify      = false;             // Alerts should send notification?
extern bool               alertsEmail       = false;             // Alerts should send an email?
extern string             soundFile         = "alert2.wav";      // Alerts sound file
extern bool               ShowText          = true;              // Show cci text?
extern string             TextDir           = "cci Text";        // Cci text id
extern int                textXPosition     = 5;                 // Cci text left to right position
extern int                textYPosition     = 30;                // Cci text up and down position
extern color              upTextcolor       = clrLimeGreen;      // Cci text up trend color   
extern color              dnTextcolor       = clrRed;            // Cci text down trend color
extern int                upFontsize        = 12;                // Cci text up trend font size
extern int                dnFontsize        = 12;                // Cci text down trend font size
extern ENUM_BASE_CORNER   textCorner        = 0;                 // Cci text corner
extern bool               Interpolate       = true;              // Interpolation used in mtf

//
//
//
//
//

double cci[];
double UpH[];
double DnH[];
double prices[];
double trend[];
string cciText;
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
      SetIndexBuffer(0,UpH); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,DnH); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,cci);
      SetIndexBuffer(3,prices);
      SetIndexBuffer(4,trend);
      SetLevelValue(0,OverBought);
      SetLevelValue(1,OverSold);

      //
      //
      //
      //
      //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = fmax(TimeFrame,_Period);
         
      //
      //
      //
      //
      //
         
   IndicatorShortName(timeFrameToString(TimeFrame)+" CCI ("+(string)CCIPeriod+")");
return(0);
}
int deinit()
{
   ObjectDelete(TextDir);
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int start()
{
    int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=fmin(Bars-counted_bars,Bars-1); 
           if (returnBars) { cci[0] = fmin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //

   if (TimeFrame==Period())
   {
      for(i=limit; i>=0; i--)
      {
         prices[i]  = getPrice(CCIPrice,Open,Close,High,Low,i);
         double avg = 0; for(int k=0; k<CCIPeriod && (i+k)<Bars; k++) avg +=      prices[i+k];      avg /= CCIPeriod;
         double dev = 0; for(int k=0; k<CCIPeriod && (i+k)<Bars; k++) dev += fabs(prices[i+k]-avg); dev /= CCIPeriod;
            if (dev!=0)
                  cci[i] = (prices[i]-avg)/(0.015*dev);
            else  cci[i] = 0;
            cciText  = (string)cci[i];
            UpH[i]   = EMPTY_VALUE;
            DnH[i]   = EMPTY_VALUE;
            trend[i] = (i<Bars-1) ? (cci[i]>0) ? 1 : (cci[i]<0) ? -1 : trend[i+1] : 0;
            if (trend[i] == 1) { UpH[i] = cci[i]; Commenter( 1); }
            if (trend[i] ==-1) { DnH[i] = cci[i]; Commenter(-1); }
            
      }
   manageAlerts();
   return(0);
   }      
   
   //
   //
   //
   //
   //
   
   limit = (int)fmax(limit,fmin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         UpH[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,CCIPrice,OverSold,OverBought,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowText,TextDir,textXPosition,textYPosition,upTextcolor,dnTextcolor,upFontsize,dnFontsize,textCorner,0,y);
         DnH[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,CCIPrice,OverSold,OverBought,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowText,TextDir,textXPosition,textYPosition,upTextcolor,dnTextcolor,upFontsize,dnFontsize,textCorner,1,y);
         cci[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,CCIPrice,OverSold,OverBought,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,ShowText,TextDir,textXPosition,textYPosition,upTextcolor,dnTextcolor,upFontsize,dnFontsize,textCorner,2,y);

         //
         //
         //
         //
         //
      
         if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;

         //
         //
         //
         //
         //

         int n,j; datetime time = iTime(NULL,TimeFrame,y);
         for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;
         for(j = 1; i+n < Bars && i+j < Bars && j < n; j++) 
         {
               cci[i+j] = cci[i] + (cci[i+n] - cci[i]) * j/n;
               if (UpH[i]!= EMPTY_VALUE) UpH[i+j] = cci[i+j];
               if (DnH[i]!= EMPTY_VALUE) DnH[i+j] = cci[i+j];
         }
    }
return(0);   
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//
//

#define priceInstances 1
double workHa[][priceInstances*4];
double getPrice(int tprice, const double& open[], const double& close[], const double& high[], const double& low[], int i, int instanceNo=0)
{
  if (tprice>=pr_haclose)
   {
      if (ArrayRange(workHa,0)!= Bars) ArrayResize(workHa,Bars); instanceNo*=4;
         int r = Bars-i-1;
         
         //
         //
         //
         //
         //
         
         double haOpen;
         if (r>0)
                haOpen  = (workHa[r-1][instanceNo+2] + workHa[r-1][instanceNo+3])/2.0;
         else   haOpen  = (open[i]+close[i])/2;
         double haClose = (open[i] + high[i] + low[i] + close[i]) / 4.0;
         double haHigh  = fmax(high[i],fmax(haOpen,haClose));
         double haLow   = fmin(low[i] ,fmin(haOpen,haClose));

         if(haOpen  <haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else                 { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                                workHa[r][instanceNo+2] = haOpen;
                                workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (tprice)
         {
            case pr_haclose:     return(haClose);
            case pr_haopen:      return(haOpen);
            case pr_hahigh:      return(haHigh);
            case pr_halow:       return(haLow);
            case pr_hamedian:    return((haHigh+haLow)/2.0);
            case pr_hamedianb:   return((haOpen+haClose)/2.0);
            case pr_hatypical:   return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:  return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:   return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
            case pr_hatbiased2:
               if (haClose>haOpen)  return(haHigh);
               if (haClose<haOpen)  return(haLow);
                                    return(haClose);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (tprice)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
      case pr_tbiased2:   
               if (close[i]>open[i]) return(high[i]);
               if (close[i]<open[i]) return(low[i]);
                                     return(close[i]);        
   }
   return(0);
}   

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (trend[whichBar] != trend[whichBar+1])
      {
          if (trend[whichBar]== 1) doAlert(whichBar,"crossed zero up");
           if (trend[whichBar]==-1) doAlert(whichBar,"crossed zero down");        
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ",timeFrameToString(_Period)+" Cci ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" Cci ",message);
          if (alertsNotify)  SendNotification(message);
          if (alertsSound)   PlaySound(soundFile);
   }
}

//
//
//
//
//

void Commenter(int Dir) 
{
    if (ShowText)
    {
      ObjectCreate(TextDir,OBJ_LABEL,0,0,0);
         ObjectSet(TextDir,OBJPROP_XDISTANCE,textXPosition);
         ObjectSet(TextDir,OBJPROP_YDISTANCE,textYPosition);
         ObjectSet(TextDir,OBJPROP_CORNER,textCorner);
         if (Dir == 1) ObjectSetText(TextDir,cciText,upFontsize,"Arial Black",upTextcolor);
         if (Dir ==-1) ObjectSetText(TextDir,cciText,dnFontsize,"Arial Black",dnTextcolor);
    }
return; 
}
